# [Imports]
from configobj import ConfigObj

import es

import cfglib
import gamethread
import langlib
import playerlib
import usermsg

# [Global varibales]

# used for the website-advert
round_count = 0
addon_path  = es.getAddonPath("notifications")

# [Classes]

# class Config: made it flexible for getting the values
class Config(cfglib.AddonCFG):
    STRING_VALUES = ("website", "funmaps_list")
    
    def __init__(self, path, indention=3):
        cfglib.AddonCFG.__init__(self, path, indention)
        
        self.text("********************************")
        self.text("* Notifications v 2.0 Settings *")
        self.text("********************************")
        self.text("\n")
        
        self.text("***************")
        self.text(" -> WEBSEITE <- ")
        self.text("***************")
        
        self.website_enable = self.cvar("notifications_website_enable", 1, "Website advert on/off")
        self.website = self.cvar("notifications_website", "http://my-web-site.com", "Name of your website here")
        
        self.text("\n")
        self.text("**********************************************")
        self.text(" -> FRIENDLY FIRE - RATES - MAP - LANGUAGE <- ")
        self.text("**********************************************")
        
        self.ff = self.cvar("notifications_ff", 1, '"ff" command on/off - if a player types it in, they see if it\'s turned on or off')
        self.rate_player = self.cvar("notifications_rate_player", 1, '"!rates" command on/off - the player sees what his rates are set to')
        self.player_rates = self.cvar("notifications_player_rates", 1, '"!showrates" command on/off - everyone sees what the rates of the user who typed it in are set to')
        self.map_player = self.cvar("notifications_map", 1, '"map" command on/off - shows which map is currently played')
        
        self.text("\n")
        self.text("*********************")
        self.text(" -> JOIN - ACTIVE <- ")
        self.text("*********************")
        
        self.player_join = self.cvar("notifications_player_join", 1, "Turns on/off if it should be shown that a player is joining the game")
        self.player_active = self.cvar("notifications_player_active", 1, "Turns on/off if it should be shown that the player is ready for battle")
        
        self.text("\n")
        self.text("***************")
        self.text(" -> FUNMAPS <- ")
        self.text("***************")
        
        self.funmaps = self.cvar("notifications_funmaps", 1, 'Turns on/off if every player should be given the "Assaultsuit" if they spawn on a fun map')
        self.funmaps_list = self.cvar("notifications_funmaps_list", ";", "If the above setting is set to 1, write in your funmaps here (split with ';')")
        
        self.text("\n")
        self.text("*********************************************************")
        self.text(" -> SELF-HIT - LAST HIT - TEAM ATTACK - KILLER-WEAPON <- ")
        self.text("*********************************************************")
        
        self.hurt_self = self.cvar("notifications_hurt_self", 1, "Turns on/off if the player should be warned, because they hurt themselves")
        self.team_attack = self.cvar("notifications_team_attack", 1, "Turns on/off if the player should be warned if they attacked team mate. Shows how many HP the mate has left, too")
        self.death_weapon = self.cvar("notifications_death_weapon", 1, "Turns on/off if the player should be told what weapon he was killed with and how many HP the attacker has left")
        
        self.text("\n")
        self.text("************")
        self.text(" -> BOMB <- ")
        self.text("************")
        
        self.bomb_drop_death = self.cvar("notifications_bomb_drop_death", 1, "Turns on/off if the Terrorists should be told that the bomber dropped the bomb because of their death")
        self.bomb_drop = self.cvar("notifications_bomb_drop", 1, "Turns on/off if the Terrorists should be told that the bomb has been thrown away")
        self.bomb_pickup = self.cvar("notifications_bomb_pickup", 1, "Turns on/off if the Terrorists should be told that the bomb has been picked up")
        self.bomb_planting = self.cvar("notifications_bomb_planting", 1, "Turns on/off if the Terrorists should be told that the bomber is planting the bomb")
        self.bomb_planted = self.cvar("notifications_bomb_has_been_planted", 1, "Turns on/off if the Terrorists should be told that the bomber has planted the bomb successfully")
        self.bomb_defusing = self.cvar("notifications_bomb_defusing", 1, "Turns on/off if the Counter-Terrorists should be told that the bomb is being defused")
        self.bomb_defused = self.cvar("notifications_bomb_has_been_defused", 1, "Turns on/off if the Counter-Terrorists should be told that the bomb has been defused successfully")
        
        self.text("\n")
        self.text("***************")
        self.text(" -> FLASHED <- ")
        self.text("***************")
        
        self.player_flash = self.cvar("notifications_flash", 1, "Turns on/off if the player should be told how long they've been flashed (in seconds)")
    
    def __getitem__(self, item):
        if item == "map":
            item = "map_player"
        
        value = str(getattr(self, item))
        
        if item == "funmaps_list":
            return value.replace(" ", "").split(";")
        if not item in self.STRING_VALUES:
            return int(value)
        
        return value

# [Class instances]
config   = Config(addon_path + "/notifications_config.cfg")
items    = ConfigObj(addon_path + "/notifications_items.ini")
language = langlib.Strings(addon_path + "/notifications_language.ini")

# [Server variables]
friendlyfire = es.ServerVar("friendly_fire")
currentmap   = es.ServerVar("eventscripts_currentmap")

# [Wrapper functions]

# tell all team-mates what's going on
def tellTeam(filtername, text):
    for userid in playerlib.getUseridList("%s,#alive" % filtername):
        es.tell(userid, "#multi", text)

# [Events]

def player_connect(ev):
    if config["player_join"]:
        es.msg("#multi", language("joinmsg", {"name": ev["name"]}))

def player_activated(ev):
    userid = int(ev["userid"])
    if not userid in players.keys():
        players[userid] = Player(userid)
    
    if config["player_active"]:
        es.msg("#multi", language("activemsg", {"name": ev["es_username"]}))

def player_spawn(ev):
    if int(ev["es_userteam"]) > 1 and str(currentmap) in config["funmaps_list"] and config["funmaps"]:
        userid = ev["userid"]
        es.server.queuecmd("es_xgive %s item_assaultsuit" % userid)
        es.tell(userid, "#multi", language("funmaps", {"map": str(currentmap)}))

def rount_start(ev):
    global round_count
    
    if config["website_enable"]:
        round_count += 1
        
        if round_count == 4:
            es.msg("#multi", language("website", {"site": config["website"]}))
            round_count = 0

def player_say(ev):
    text = ev["text"].strip()
    
    if text in ("ff", "map") and config[text]:
        if text == "ff":
            if int(friendlyfire):
                es.msg("#multi", language("ff on"))
            else:
                es.msg("#multi", language("ff off"))
        else:
            es.tell(ev["userid"], '#multi', language("map", {"curmap": str(currentmap)}))

def player_blind(ev):
    if config["player_flash"]:
        userid   = ev["userid"]
        
        duration = float(es.getplayerprop(userid, "CCSPlayer.m_flFlashDuration"))
        gamethread.delayed(duration, usermsg.hudhint, (userid, language("flash duration", {"duration": "%0.2f" % duration})))

def player_hurt(ev):
    userid   = int(ev["userid"])
    attacker = int(ev["attacker"])
    name     = ev["es_username"]
    dmg      = int(ev["dmg_health"])
    dead     = int(ev["es_userdead"])
    
    try:
        if int(ev["es_attackerteam"]) == int(ev["es_userteam"]):
            if dead:
                es.tell(userid, "#multi", language("team mate down", {"name": name, "dmg": dmg}))
            else:
                es.tell(userid, "#multi", language("team attack", {"name": name, "health": ev["health"]}))
    except ValueError:
        if config["hurt_self"]:
            es.tell(userid, "#multi", language("selfhit"))

def player_death(ev):
    userid = ev["userid"]
    weapon = ev["weapon"]
    health = int(ev["es_attackerhealth"]) # needs to be saved...
    
    if ev["attacker"] != userid:
        es.tell(userid, "#multi", language("attacker + weapon", {"name": ev["es_attackername"], "weapon": items[weapon]}))
        gamethread.queue(es.tell, (userid, "#multi", language("attacker health", {"health": health})))
    else:
        if weapon == "hegrenade":
            es.tell(userid, "#multi", language("nade selfkill"))
        else:
            es.tell(userid, "#multi", language("selfkill"))

def bomb_dropped(ev):
    name = ev["es_username"]
    
    if config["bomb_drop_death"] and not int(ev["es_userhealth"]):
        tellTeam("#t", language("killed bomber", {"name": name}))
    elif config["bomb_drop"]:
        tellTeam("#t", language("bomb thrown away", {"name": name}))

def bomb_pickup(ev):
    if config["bomb_pickup"]:
        tellTeam("#t", language("bomb picked up", {"name": ev["es_username"]}))

def bomb_beginplant(ev):
    if config["bomb_planting"]:
        tellTeam("#t", language("bomb planting", {"name": ev["es_username"]}))

def bomb_planted(ev):
    if config["bomb_planted"]:
        tellTeam("#t", language("bomb planted", {"name": ev["es_username"]}))

def bomb_begindefuse(ev):
    if config["bomb_defusing"]:
        tellTeam("#ct", language("bomb defusing", {"name": ev["es_username"]}))

def bomb_defused(ev):
    if config["bomb_defused"]:
        tellTeam("#ct", language("bomb defused", {"name": ev["es_username"]}))

# [Say-Commands]

def saycommand_rates():
    userid = es.getcmduserid()
    
    es.tell(userid, "#multi", language("rates are"))
    gamethread.delayed(.2, es.tell, (userid, "#multi", "#green[#lightgreenNotifications#green] #lightgreenrate: #green%s" % es.getclientvar(userid, "rate")))
    gamethread.delayed(.2, es.tell, (userid, "#multi", "#green[#lightgreenNotifications#green] #lightgreencl_cmdrate: #green%s" % es.getclientvar(userid, "cl_cmdrate")))
    gamethread.delayed(.2, es.tell, (userid, "#multi", "#green[#lightgreenNotifications#green] #lightgreencl_updaterate: #green%s" % es.getclientvar(userid, "cl_updaterate")))

def saycommand_showrates():
    userid = es.getcmduserid()
    
    es.msg("#multi", language("show rates", {"name": es.getplayername(userid)}))
    gamethread.delayed(.2, es.msg, ("#multi", "#green[#lightgreenNotifications#green] #lightgreenrate: #green%s" % es.getclientvar(userid, "rate")))
    gamethread.delayed(.2, es.msg, ("#multi", "#green[#lightgreenNotifications#green] #lightgreencl_cmdrate: #green%s" % es.getclientvar(userid, "cl_cmdrate")))
    gamethread.delayed(.2, es.msg, ("#multi", "#green[#lightgreenNotifications#green] #lightgreencl_updaterate: #green%s" % es.getclientvar(userid, "cl_updaterate")))

# [Client Command Filter]
def clientCommandFilter(userid, args):
    if args[0] == "buy":
        es.tell(userid, "#multi", language("bought item", {"item": args[1]}))
    return True

# [Load & Unload]

def load():
    config.write()
    config.execute()
    
    es.addons.registerClientCommandFilter(clientCommandFilter)
    
    if not es.exists("saycommand", "!rates"):
        es.regsaycmd("!rates", "notifications/saycommand_rates")
    if not es.exists("saycommand", "!showrates"):
        es.regsaycmd("!showrates", "notifications/saycommand_showrates")

def unload():
    es.addons.unregisterClientCommandFilter(clientCommandFilter)
    
    es.unregsaycmd("!rates")
    es.unregsaycmd("!showrates")
